@@ -216,19 +216,19 @@ CCMD(am_togglefollow)
 	am_followplayer = !am_followplayer;
 	if (primaryLevel && primaryLevel->automap)
 		primaryLevel->automap->ResetFollowLocation();
-	Printf("%s\n", GStrings(am_followplayer ? "AMSTR_FOLLOWON" : "AMSTR_FOLLOWOFF"));
+	Printf("%s\n", GStrings.GetString(am_followplayer ? "AMSTR_FOLLOWON" : "AMSTR_FOLLOWOFF"));
 }
 
 CCMD(am_togglegrid)
 {
 	am_showgrid = !am_showgrid;
-	Printf("%s\n", GStrings(am_showgrid ? "AMSTR_GRIDON" : "AMSTR_GRIDOFF"));
+	Printf("%s\n", GStrings.GetString(am_showgrid ? "AMSTR_GRIDON" : "AMSTR_GRIDOFF"));
 }
 
 CCMD(am_toggletexture)
 {
 	am_textured = !am_textured;
-	Printf("%s\n", GStrings(am_textured ? "AMSTR_TEXON" : "AMSTR_TEXOFF"));
+	Printf("%s\n", GStrings.GetString(am_textured ? "AMSTR_TEXON" : "AMSTR_TEXOFF"));
 }
 
 CCMD(am_setmark)
@@ -238,7 +238,7 @@ CCMD(am_setmark)
 		int m = primaryLevel->automap->addMark();
 		if (m >= 0)
 		{
-			Printf("%s %d\n", GStrings("AMSTR_MARKEDSPOT"), m);
+			Printf("%s %d\n", GStrings.GetString("AMSTR_MARKEDSPOT"), m);
 		}
 	}
 }
@@ -247,7 +247,7 @@ CCMD(am_clearmarks)
 {
 	if (primaryLevel && primaryLevel->automap && primaryLevel->automap->clearMarks())
 	{
-		Printf("%s\n", GStrings("AMSTR_MARKSCLEARED"));
+		Printf("%s\n", GStrings.GetString("AMSTR_MARKSCLEARED"));
 	}
 }
 
@@ -2624,7 +2624,7 @@ void DAutomap::drawWalls (bool allmap)
 				}
 				else if (line.flags & ML_SECRET)
 				{ // secret door
-					if (am_cheat != 0 && line.backsector != nullptr)
+					if (am_cheat != 0 && am_cheat < 4 && line.backsector != nullptr)
 						drawMline(&l, AMColors.SecretWallColor);
 					else
 						drawMline(&l, AMColors.WallColor);
@@ -2816,7 +2816,7 @@ void DAutomap::drawPlayers ()
 		int numarrowlines;
 
 		double vh = players[consoleplayer].viewheight;
-		DVector2 pos = players[consoleplayer].camera->InterpolatedPosition(r_viewpoint.TicFrac).XY();
+		DVector2 pos = players[consoleplayer].mo->InterpolatedPosition(r_viewpoint.TicFrac).XY();
 		pt.x = pos.X;
 		pt.y = pos.Y;
 		if (am_rotate == 1 || (am_rotate == 2 && viewactive))
@@ -2826,7 +2826,7 @@ void DAutomap::drawPlayers ()
 		}
 		else
 		{
-			angle = players[consoleplayer].camera->InterpolatedAngles(r_viewpoint.TicFrac).Yaw;
+			angle = players[consoleplayer].mo->InterpolatedAngles(r_viewpoint.TicFrac).Yaw;
 		}
 		
 		if (am_cheat != 0 && CheatMapArrow.Size() > 0)
@@ -2910,11 +2910,18 @@ void DAutomap::drawKeys ()
 	mpoint_t p;
 	DAngle	 angle;
 
-	auto it = Level->GetThinkerIterator<AActor>(NAME_Key);
+	auto it = Level->GetThinkerIterator<AActor>(NAME_Inventory);
 	AActor *key;
 
 	while ((key = it.Next()) != nullptr)
 	{
+		auto cls = key->GetClass();
+		if (cls->IsDescendantOf(NAME_DehackedPickup))
+		{
+			cls = cls->ActorInfo()->Replacee;
+		}
+		if (!cls->IsDescendantOf(NAME_Key)) continue;
+
 		DVector3 pos = key->PosRelative(MapPortalGroup);
 		p.x = pos.X;
 		p.y = pos.Y;
