@@ -107,6 +107,10 @@ public:
 	virtual bool GetEnabled();
 	virtual void SetEnabled(bool enabled);
 
+	bool AllowsEnabledInBackground() { return false; }
+	bool GetEnabledInBackground() { return false; }
+	void SetEnabledInBackground(bool enabled) {}
+
 	virtual void SetDefaultConfig();
 	virtual FString GetIdentifier();
 
