@@ -65,6 +65,8 @@
 #endif
 #endif
 
+extern bool AppActive;
+
 // TYPES -------------------------------------------------------------------
 
 typedef DWORD (WINAPI *XInputGetStateType)(DWORD index, XINPUT_STATE *state);
@@ -105,6 +107,10 @@ public:
 	bool GetEnabled();
 	void SetEnabled(bool enabled);
 
+	bool AllowsEnabledInBackground() { return true; }
+	bool GetEnabledInBackground() { return EnabledInBackground; }
+	void SetEnabledInBackground(bool enabled) { EnabledInBackground = enabled; }
+
 	void SetDefaultConfig();
 	FString GetIdentifier();
 
@@ -142,6 +148,7 @@ protected:
 	int LastButtons;
 	bool Connected;
 	bool Enabled;
+	bool EnabledInBackground;
 
 	void Attached();
 	void Detached();
@@ -744,7 +751,10 @@ void FXInputManager::ProcessInput()
 {
 	for (int i = 0; i < XUSER_MAX_COUNT; ++i)
 	{
-		Devices[i]->ProcessInput();
+		if(AppActive || Devices[i]->GetEnabledInBackground())
+		{
+			Devices[i]->ProcessInput();
+		}
 	}
 }
 
