@@ -670,7 +670,7 @@ DEFINE_ACTION_FUNCTION_NATIVE(FFont, GetBottomAlignOffset, GetBottomAlignOffset)
 
 static int StringWidth(FFont *font, const FString &str, int localize)
 {
-	const char *txt = (localize && str[0] == '$') ? GStrings(&str[1]) : str.GetChars();
+	const char *txt = (localize && str[0] == '$') ? GStrings.GetString(&str[1]) : str.GetChars();
 	return font->StringWidth(txt);
 }
 
@@ -684,7 +684,7 @@ DEFINE_ACTION_FUNCTION_NATIVE(FFont, StringWidth, StringWidth)
 
 static int GetMaxAscender(FFont* font, const FString& str, int localize)
 {
-	const char* txt = (localize && str[0] == '$') ? GStrings(&str[1]) : str.GetChars();
+	const char* txt = (localize && str[0] == '$') ? GStrings.GetString(&str[1]) : str.GetChars();
 	return font->GetMaxAscender(txt);
 }
 
@@ -698,7 +698,7 @@ DEFINE_ACTION_FUNCTION_NATIVE(FFont, GetMaxAscender, GetMaxAscender)
 
 static int CanPrint(FFont *font, const FString &str, int localize)
 {
-	const char *txt = (localize && str[0] == '$') ? GStrings(&str[1]) : str.GetChars();
+	const char *txt = (localize && str[0] == '$') ? GStrings.GetString(&str[1]) : str.GetChars();
 	return font->CanPrint(txt);
 }
 
@@ -770,6 +770,26 @@ DEFINE_ACTION_FUNCTION_NATIVE(FFont, GetDisplayTopOffset, GetDisplayTopOffset)
 	ACTION_RETURN_FLOAT(GetDisplayTopOffset(self, code));
 }
 
+static int GetChar(FFont* font, int c)
+{
+	int texc = 0;
+	auto getch = font->GetChar(c, CR_UNDEFINED, nullptr);
+	if (getch)
+		texc = getch->GetID().GetIndex();
+	return texc;
+}
+
+DEFINE_ACTION_FUNCTION_NATIVE(FFont, GetChar, ::GetChar)
+{
+	PARAM_SELF_STRUCT_PROLOGUE(FFont);
+	PARAM_INT(mchar);
+
+	if (numret > 0) ret[0].SetInt(::GetChar(self, mchar));
+	if (numret > 1) ret[1].SetInt(self->GetCharWidth(mchar));
+	return min(2, numret);
+}
+
+
 //==========================================================================
 //
 // file system
@@ -882,6 +902,27 @@ DEFINE_ACTION_FUNCTION(_CVar, GetString)
 	ACTION_RETURN_STRING(v.String);
 }
 
+DEFINE_ACTION_FUNCTION(_CVar, GetDefaultInt)
+{
+	PARAM_SELF_STRUCT_PROLOGUE(FBaseCVar);
+	auto v = self->GetGenericRepDefault(CVAR_Int);
+	ACTION_RETURN_INT(v.Int);
+}
+
+DEFINE_ACTION_FUNCTION(_CVar, GetDefaultFloat)
+{
+	PARAM_SELF_STRUCT_PROLOGUE(FBaseCVar);
+	auto v = self->GetGenericRepDefault(CVAR_Float);
+	ACTION_RETURN_FLOAT(v.Float);
+}
+
+DEFINE_ACTION_FUNCTION(_CVar, GetDefaultString)
+{
+	PARAM_SELF_STRUCT_PROLOGUE(FBaseCVar);
+	auto v = self->GetGenericRepDefault(CVAR_String);
+	ACTION_RETURN_STRING(v.String);
+}
+
 DEFINE_ACTION_FUNCTION(_CVar, SetInt)
 {
 	// Only menus are allowed to change CVARs.
@@ -1142,6 +1183,17 @@ DEFINE_ACTION_FUNCTION(_Console, PrintfEx)
 	return 0;
 }
 
+DEFINE_ACTION_FUNCTION(_Console, DebugPrintf)
+{
+	PARAM_PROLOGUE;
+	PARAM_INT(debugLevel);
+	PARAM_VA_POINTER(va_reginfo);
+
+	FString s = FStringFormat(VM_ARGS_NAMES, 1);
+	DPrintf(debugLevel, "%s\n", s.GetChars());
+	return 0;
+}
+
 static void StopAllSounds()
 {
 	soundEngine->StopAllChannels();
@@ -1675,7 +1727,7 @@ DEFINE_ACTION_FUNCTION(DScriptScanner, ScriptError)
 {
 	PARAM_SELF_PROLOGUE(DScriptScanner);
 
-	FString s = FStringFormat(VM_ARGS_NAMES);
+	FString s = FStringFormat(VM_ARGS_NAMES, 1);
 	self->wrapped.ScriptError("%s", s.GetChars());
 	return 0;
 }
@@ -1684,7 +1736,7 @@ DEFINE_ACTION_FUNCTION(DScriptScanner, ScriptMessage)
 {
 	PARAM_SELF_PROLOGUE(DScriptScanner);
 
-	FString s = FStringFormat(VM_ARGS_NAMES);
+	FString s = FStringFormat(VM_ARGS_NAMES, 1);
 	self->wrapped.ScriptMessage("%s", s.GetChars());
 	return 0;
 }
@@ -1765,3 +1817,16 @@ DEFINE_FIELD_NAMED_X(ScriptScanner, DScriptScanner, wrapped.Number, Number);
 DEFINE_FIELD_NAMED_X(ScriptScanner, DScriptScanner, wrapped.End, End);
 DEFINE_FIELD_NAMED_X(ScriptScanner, DScriptScanner, wrapped.Crossed, Crossed);
 DEFINE_FIELD_NAMED_X(ScriptScanner, DScriptScanner, wrapped.ParseError, ParseError);
+
+static int ValidateNameIndex(int index)
+{
+	return FName::IsValidName(index) ? index : 0;
+}
+
+DEFINE_ACTION_FUNCTION_NATIVE(DObject, ValidateNameIndex, ValidateNameIndex)
+{
+	PARAM_PROLOGUE;
+	PARAM_INT(index);
+
+	ACTION_RETURN_INT(ValidateNameIndex(index));
+}
