@@ -372,6 +372,13 @@ FStartScreen* GetGameStartScreen(int max_progress)
 	return nullptr;
 }
 
+FStartScreen::~FStartScreen()
+{
+	if (StartupTexture) delete StartupTexture;
+	if (HeaderTexture) delete HeaderTexture;
+	if (NetTexture) delete NetTexture;
+}
+
 //==========================================================================
 //
 // ST_Util_ClearBlock
@@ -607,7 +614,7 @@ bool FStartScreen::NetInit(const char* message, int numplayers)
 {
 	NetMaxPos = numplayers;
 	NetCurPos = 0;
-	NetMessageString.Format("%s %s", message, GStrings("TXT_NET_PRESSESC"));
+	NetMessageString.Format("%s %s", message, GStrings.GetString("TXT_NET_PRESSESC"));
 	NetProgress(1);	// You always know about yourself
 	return true;
 }
@@ -692,8 +699,8 @@ void FStartScreen::Render(bool force)
 		twod->OnFrameDone();
 	}
 	auto newtime = I_msTime();
-	if ((newtime - nowtime) * 2.0 > minwaittime) // slow down drawing the start screen if we're on a slow GPU!
-		minwaittime = (newtime - nowtime) * 2.0;
+	if ((newtime - nowtime) * 2 > minwaittime) // slow down drawing the start screen if we're on a slow GPU!
+		minwaittime = (newtime - nowtime) * 2;
 }
 
 FImageSource* CreateStartScreenTexture(FBitmap& srcdata);
