@@ -59,6 +59,10 @@ EXTERN_CVAR(Bool, autoloadbrightmaps)
 EXTERN_CVAR(Bool, autoloadwidescreen)
 EXTERN_CVAR(String, language)
 
+CVAR(Int, i_loadsupportwad, 1, CVAR_ARCHIVE|CVAR_GLOBALCONFIG) // 0=never, 1=singleplayer only, 2=always
+
+bool foundprio = false; // global to prevent iwad box from appearing
+
 //==========================================================================
 //
 // Parses IWAD definitions
@@ -111,6 +115,12 @@ void FIWadManager::ParseIWadInfo(const char *fn, const char *data, int datasize,
 						iwad->prio = sc.Number;
 					}
 				}
+				else if (sc.Compare("SupportWAD"))
+				{
+					sc.MustGetStringName("=");
+					sc.MustGetString();
+					iwad->SupportWAD = sc.String;
+				}
 				else if (sc.Compare("Config"))
 				{
 					sc.MustGetStringName("=");
@@ -138,6 +148,10 @@ void FIWadManager::ParseIWadInfo(const char *fn, const char *data, int datasize,
 				{
 					iwad->nokeyboardcheats = true;
 				}
+				else if (sc.Compare("SkipBexStringsIfLanguage"))
+				{
+					iwad->SkipBexStringsIfLanguage = true;
+				}
 				else if (sc.Compare("Compatibility"))
 				{
 					sc.MustGetStringName("=");
@@ -558,6 +572,18 @@ void FIWadManager::ValidateIWADs()
 
 static bool havepicked = false;
 
+
+FString FIWadManager::IWADPathFileSearch(const FString &file)
+{
+	for(const FString &path : mSearchPaths)
+	{
+		FString f = path + "/" + file;
+		if(FileExists(f)) return f;
+	}
+
+	return "";
+}
+
 int FIWadManager::IdentifyVersion (std::vector<std::string>&wadfiles, const char *iwad, const char *zdoom_wad, const char *optional_wad)
 {
 	const char *iwadparm = Args->CheckValue ("-iwad");
@@ -673,6 +699,7 @@ int FIWadManager::IdentifyVersion (std::vector<std::string>&wadfiles, const char
 				picks.Clear();
 				picks.Push(found);
 				pickedprio = mIWadInfos[found.mInfoIndex].prio;
+				foundprio = true;
 			}
 		}
 	}
@@ -727,8 +754,10 @@ int FIWadManager::IdentifyVersion (std::vector<std::string>&wadfiles, const char
 	}
 	int pick = 0;
 
-	// We got more than one so present the IWAD selection box.
-	if (picks.Size() > 1)
+	// Present the IWAD selection box.
+	bool alwaysshow = (queryiwad && !Args->CheckParm("-iwad") && !foundprio);
+
+	if (alwaysshow || picks.Size() > 1)
 	{
 		// Locate the user's prefered IWAD, if it was found.
 		if (defaultiwad[0] != '\0')
@@ -743,7 +772,7 @@ int FIWadManager::IdentifyVersion (std::vector<std::string>&wadfiles, const char
 				}
 			}
 		}
-		if (picks.Size() > 1)
+		if (alwaysshow || picks.Size() > 1)
 		{
 			if (!havepicked)
 			{
@@ -762,9 +791,18 @@ int FIWadManager::IdentifyVersion (std::vector<std::string>&wadfiles, const char
 				if (autoloadbrightmaps) flags |= 4;
 				if (autoloadwidescreen) flags |= 8;
 
-				pick = I_PickIWad(&wads[0], (int)wads.Size(), queryiwad, pick, flags);
+				FString extraArgs;
+
+				pick = I_PickIWad(&wads[0], (int)wads.Size(), queryiwad, pick, flags, extraArgs);
 				if (pick >= 0)
 				{
+					extraArgs.StripLeftRight();
+
+					if(extraArgs.Len() > 0)
+					{
+						Args->AppendArgsString(extraArgs);
+					}
+
 					disableautoload = !!(flags & 1);
 					autoloadlights = !!(flags & 2);
 					autoloadbrightmaps = !!(flags & 4);
@@ -787,11 +825,11 @@ int FIWadManager::IdentifyVersion (std::vector<std::string>&wadfiles, const char
 	D_AddFile (wadfiles, zdoom_wad, true, -1, GameConfig);
 
 	// [SP] Load non-free assets if available. This must be done before the IWAD.
-	int iwadnum;
-	if (D_AddFile(wadfiles, optional_wad, true, -1, GameConfig))
-		iwadnum = 2;
-	else
-		iwadnum = 1;
+	int iwadnum = 1;
+	if (optional_wad && D_AddFile(wadfiles, optional_wad, true, -1, GameConfig))
+	{
+		iwadnum++;
+	}
 
 	fileSystem.SetIwadNum(iwadnum);
 	if (picks[pick].mRequiredPath.IsNotEmpty())
@@ -803,6 +841,22 @@ int FIWadManager::IdentifyVersion (std::vector<std::string>&wadfiles, const char
 	fileSystem.SetMaxIwadNum(iwadnum);
 
 	auto info = mIWadInfos[picks[pick].mInfoIndex];
+
+	if(info.SupportWAD.IsNotEmpty())
+	{
+		bool wantsnetgame = (Args->CheckParm("-join") || Args->CheckParm("-host"));
+
+		if ((!wantsnetgame && i_loadsupportwad == 1) || (i_loadsupportwad == 2))
+		{
+			FString supportWAD = IWADPathFileSearch(info.SupportWAD);
+
+			if(supportWAD.IsNotEmpty())
+			{
+				D_AddFile(wadfiles, supportWAD.GetChars(), true, -1, GameConfig);
+			}
+		}
+	}
+
 	// Load additional resources from the same directory as the IWAD itself.
 	for (unsigned i=0; i < info.Load.Size(); i++)
 	{
