@@ -791,6 +791,7 @@ public:
 				th->friendlyseeblocks = CheckInt(key);
 				break;
 
+			case NAME_light_softshadowradius:
 			case NAME_lm_suncolor:
 			case NAME_lm_sampledist:
 				CHECK_N(Zd | Zdt)
@@ -1002,6 +1003,10 @@ public:
 			// This switch contains all keys of the UDMF base spec which only apply to Hexen format specials
 			if (!isTranslated) switch (key.GetIndex())
 			{
+			case NAME_Walking:
+				Flag(ld->activation, SPAC_Walking, key);
+				continue;
+
 			case NAME_Playercross:
 				Flag(ld->activation, SPAC_Cross, key); 
 				continue;
@@ -1166,7 +1171,7 @@ public:
 				ld->healthgroup = CheckInt(key);
 				break;
 
-			case NAME_lm_sampledist_line:
+			case NAME_lm_sampledist:
 			case NAME_lm_sampledist_top:
 			case NAME_lm_sampledist_mid:
 			case NAME_lm_sampledist_bot:
@@ -1516,7 +1521,7 @@ public:
 					sd->Flags |= WALLF_EXTCOLOR;
 				break;
 
-			case NAME_lm_sampledist_line:
+			case NAME_lm_sampledist:
 			case NAME_lm_sampledist_top:
 			case NAME_lm_sampledist_mid:
 			case NAME_lm_sampledist_bot:
@@ -1974,6 +1979,14 @@ public:
 					Flag(sec->Flags, SECF_HAZARD, key);
 					break;
 
+				case NAME_hurtmonsters:
+					Flag(sec->MoreFlags, SECMF_HURTMONSTERS, key);
+					break;
+
+				case NAME_harminair:
+					Flag(sec->MoreFlags, SECMF_HARMINAIR, key);
+					break;
+
 				case NAME_floorterrain:
 					sec->terrainnum[sector_t::floor] = P_FindTerrain(CheckString(key));
 					break;
