@@ -62,7 +62,7 @@
 #include "doommenu.h"
 #include "g_game.h"
 
-static FRandom pr_randomspeech("RandomSpeech");
+static FCRandom pr_randomspeech("RandomSpeech");
 
 static int ConversationMenuY;
 
@@ -550,7 +550,7 @@ static void HandleReply(player_t *player, bool isconsole, int nodenum, int reply
 			}
 		
 			if (reply->GiveType->IsDescendantOf("SlideshowStarter"))
-				G_StartSlideshow(primaryLevel, NAME_None);
+				G_StartSlideshow(primaryLevel, NAME_None, FSTATE_InLevel);
 		}
 		else
 		{
@@ -589,7 +589,7 @@ static void HandleReply(player_t *player, bool isconsole, int nodenum, int reply
 		const char *log = reply->LogString.GetChars();
 		if (log[0] == '$')
 		{
-			log = GStrings(log + 1);
+			log = GStrings.GetString(log + 1);
 		}
 
 		player->SetLogText(log);
@@ -713,7 +713,7 @@ static void TerminalResponse (const char *str)
 		// handle string table replacement
 		if (str[0] == '$')
 		{
-			str = GStrings(str + 1);
+			str = GStrings.GetString(str + 1);
 		}
 
 		if (StatusBar != NULL)
