@@ -262,8 +262,8 @@ enum ActorFlag4
 	MF4_STRIFEDAMAGE	= 0x00000100,	// Strife projectiles only do up to 4x damage, not 8x
 
 	MF4_CANUSEWALLS		= 0x00000200,	// Can activate 'use' specials
-	MF4_MISSILEMORE		= 0x00000400,	// increases the chance of a missile attack
-	MF4_MISSILEEVENMORE	= 0x00000800,	// significantly increases the chance of a missile attack
+	//		= 0x00000400,
+	//		= 0x00000800,
 	MF4_FORCERADIUSDMG	= 0x00001000,	// if put on an object it will override MF3_NORADIUSDMG
 	MF4_DONTFALL		= 0x00002000,	// Doesn't have NOGRAVITY disabled when dying.
 	MF4_SEESDAGGERS		= 0x00004000,	// This actor can see you striking with a dagger
@@ -442,7 +442,11 @@ enum ActorFlag9
 	MF9_DOSHADOWBLOCK			= 0x00000002,	// [inkoalawetrust] Should the monster look for SHADOWBLOCK actors ?
 	MF9_SHADOWBLOCK				= 0x00000004,	// [inkoalawetrust] Actors in the line of fire with this flag trigger the MF_SHADOW aiming penalty.
 	MF9_SHADOWAIMVERT			= 0x00000008,	// [inkoalawetrust] Monster aim is also offset vertically when aiming at shadow actors.
-	MF9_DECOUPLEDANIMATIONS	= 0x00000010,	// [RL0] Decouple model animations from states
+	MF9_DECOUPLEDANIMATIONS		= 0x00000010,	// [RL0] Decouple model animations from states
+	MF9_NOSECTORDAMAGE			= 0x00000020,	// [inkoalawetrust] Actor ignores any sector-based damage (i.e damaging floors, NOT crushers)
+	MF9_ISPUFF					= 0x00000040,	// [AA] Set on actors by P_SpawnPuff
+	MF9_FORCESECTORDAMAGE		= 0x00000080,	// [inkoalawetrust] Actor ALWAYS takes hurt floor damage if there's any. Even if the floor doesn't have SECMF_HURTMONSTERS.
+	MF9_NOAUTOOFFSKULLFLY		= 0x00000100,	// Don't automatically disable MF_SKULLFLY if velocity is 0.
 };
 
 // --- mobj.renderflags ---
@@ -501,6 +505,9 @@ enum ActorRenderFlag2
 	RF2_FLIPSPRITEOFFSETX		= 0x0010,
 	RF2_FLIPSPRITEOFFSETY		= 0x0020,
 	RF2_CAMFOLLOWSPLAYER		= 0x0040,	// Matches the cam's base position and angles to the main viewpoint.
+	RF2_ISOMETRICSPRITES		= 0x0080,
+	RF2_SQUAREPIXELS			= 0x0100,	// apply +ROLLSPRITE scaling math so that non rolling sprites get the same scaling
+	RF2_STRETCHPIXELS			= 0x0200,	// don't apply SQUAREPIXELS for ROLLSPRITES
 };
 
 // This translucency value produces the closest match to Heretic's TINTTAB.
@@ -542,6 +549,8 @@ enum ActorBounceFlag
 	BOUNCE_BounceOnUnrips = 1<<16,	// projectile bounces on actors with DONTRIP
 	BOUNCE_NotOnSky = 1<<17,		// Don't bounce on sky floors / ceilings / walls
 	BOUNCE_DEH = 1<<18,				// Flag was set through Dehacked.
+	BOUNCE_KeepAngle = 1<<19,		// Don't change yaw when bouncing off a surface.
+	BOUNCE_ModifyPitch = 1<<20,		// Change pitch when bouncing off a surface.
 
 	BOUNCE_TypeMask = BOUNCE_Walls | BOUNCE_Floors | BOUNCE_Ceilings | BOUNCE_Actors | BOUNCE_AutoOff | BOUNCE_HereticType | BOUNCE_MBF,
 
@@ -691,26 +700,10 @@ struct FDropItem
 
 enum EViewPosFlags // [MC] Flags for SetViewPos.
 {
-	VPSF_ABSOLUTEOFFSET =	1 << 1,			// Don't include angles.
-	VPSF_ABSOLUTEPOS =		1 << 2,			// Use absolute position.
-};
-
-enum EAnimOverrideFlags
-{
-	ANIMOVERRIDE_NONE	= 1 << 0, // no animation
-	ANIMOVERRIDE_LOOP	= 1 << 1, // animation loops, otherwise it stays on the last frame once it ends
-};
-
-struct AnimOverride
-{
-	int firstFrame;
-	int lastFrame;
-	int loopFrame;
-	double startFrame;
-	int flags = ANIMOVERRIDE_NONE;
-	float framerate;
-	double startTic; // when the current animation started (changing framerates counts as restarting) (or when animation starts if interpolating from previous animation)
-	double switchTic; // when the animation was changed -- where to interpolate the switch from
+	VPSF_ABSOLUTEOFFSET =		1 << 1,			// Don't include angles.
+	VPSF_ABSOLUTEPOS =			1 << 2,			// Use absolute position.
+	VPSF_ALLOWOUTOFBOUNDS =	1 << 3,			// Allow viewpoint to go out of bounds (hardware renderer only).
+	VPSF_ORTHOGRAPHIC =		1 << 4,			// Use orthographic projection (hardware renderer only).
 };
 
 struct ModelOverride
@@ -748,8 +741,8 @@ public:
 	int							overrideFlagsSet;
 	int							overrideFlagsClear;
 
-	AnimOverride curAnim;
-	AnimOverride prevAnim; // used for interpolation when switching anims
+	ModelAnim curAnim;
+	ModelAnimFrame prevAnim; // used for interpolation when switching anims
 
 	DActorModelData() = default;
 	virtual void Serialize(FSerializer& arc) override;
@@ -766,14 +759,11 @@ public:
 	int			Flags;
 
 	// Functions
-	DViewPosition()
-	{
-		Offset = { 0,0,0 };
-		Flags = 0;
-	}
-
 	void Set(DVector3 &off, int f = -1)
 	{
+		ZeroSubnormalsF(off.X);
+		ZeroSubnormalsF(off.Y);
+		ZeroSubnormalsF(off.Z);
 		Offset = off;
 
 		if (f > -1)
@@ -784,6 +774,8 @@ public:
 	{
 		return Offset.isZero();
 	}
+
+	void Serialize(FSerializer& arc) override;
 };
 
 const double MinVel = EQUAL_EPSILON;
@@ -874,13 +866,13 @@ public:
 	void Howl ();
 
 	// plays bouncing sound
-	void PlayBounceSound(bool onfloor);
+	void PlayBounceSound(bool onfloor, double volume);
 
 	// plays pushing sound
 	void PlayPushSound();
 
 	// Called when an actor with MF_MISSILE and MF2_FLOORBOUNCE hits the floor
-	bool FloorBounceMissile (secplane_t &plane);
+	bool FloorBounceMissile (secplane_t &plane, bool is3DFloor);
 
 	// Called by RoughBlockCheck
 	bool IsOkayToAttack (AActor *target);
@@ -899,7 +891,7 @@ public:
 	// Called when bouncing to allow for custom behavior.
 	// Returns -1 for normal behavior, 0 to stop, and 1 to keep going.
 	// (virtual on the script side only)
-	int SpecialBounceHit(AActor* bounceMobj, line_t* bounceLine, secplane_t* bouncePlane);
+	int SpecialBounceHit(AActor* bounceMobj, line_t* bounceLine, secplane_t* bouncePlane, bool is3DFloor);
 
 	// Returns true if it's okay to switch target to "other" after being attacked by it.
 	bool CallOkayToSwitchTarget(AActor *other);
@@ -1119,6 +1111,8 @@ public:
 	DAngle			SpriteAngle;
 	DAngle			SpriteRotation;
 	DVector2		AutomapOffsets;		// Offset the actors' sprite view on the automap by these coordinates.
+	float			isoscaleY;				// Y-scale to compensate for Y-billboarding for isometric sprites
+	float			isotheta;				// Rotation angle to compensate for Y-billboarding for isometric sprites
 	DRotator		Angles;
 	DRotator		ViewAngles;			// Angle offsets for cameras
 	TObjPtr<DViewPosition*> ViewPos;			// Position offsets for cameras
@@ -1164,7 +1158,6 @@ public:
 	double			Speed;
 	double			FloatSpeed;
 	TObjPtr<DActorModelData*>		modelData;
-	TObjPtr<DBoneComponents*>		boneComponentData;
 
 // interaction info
 	FBlockNode		*BlockNode;			// links in blocks (if needed)
@@ -1239,6 +1232,7 @@ public:
 	TObjPtr<AActor*>	alternative;	// (Un)Morphed actors stored here. Those with the MF_UNMORPHED flag are the originals.
 	TObjPtr<AActor*>	tracer;			// Thing being chased/attacked for tracers
 	TObjPtr<AActor*>	master;			// Thing which spawned this one (prevents mutual attacks)
+	TObjPtr<AActor*>	damagesource;	// [AA] Thing that fired a hitscan using this actor as a puff
 
 	int				tid;			// thing identifier
 	int				special;		// special
@@ -1260,6 +1254,7 @@ public:
 									// but instead tries to come closer for a melee attack.
 									// This is not the same as meleerange
 	double			maxtargetrange;	// any target farther away cannot be attacked
+	double			missilechancemult; // distance multiplier for CheckMeleeRange, formerly done with MISSILE(EVEN)MORE flags.
 	double			bouncefactor;	// Strife's grenades use 50%, Hexen's Flechettes 70.
 	double			wallbouncefactor;	// The bounce factor for walls can be different.
 	double			Gravity;		// [GRB] Gravity factor
@@ -1312,6 +1307,7 @@ public:
 	uint8_t FloatBobPhase;
 	uint8_t FriendPlayer;				// [RH] Player # + 1 this friendly monster works for (so 0 is no player, 1 is player 0, etc)
 	double FloatBobStrength;
+	double FloatBobFactor;
 	PalEntry BloodColor;
 	FTranslationID BloodTranslation;
 
@@ -1402,6 +1398,7 @@ public:
 	bool IsMapActor();
 	bool SetState (FState *newstate, bool nofunction=false);
 	void SplashCheck();
+	void PlayDiveOrSurfaceSounds(int oldlevel = 0);
 	bool UpdateWaterLevel (bool splash=true);
 	bool isFast();
 	bool isSlow();
@@ -1530,9 +1527,13 @@ public:
 	{
 		return Z() + Height;
 	}
+	double CenterOffset() const
+	{
+		return Height / 2;
+	}
 	double Center() const
 	{
-		return Z() + Height/2;
+		return Z() + CenterOffset();
 	}
 	void SetZ(double newz, bool moving = true)
 	{
@@ -1609,6 +1610,11 @@ public:
 		Vel.Y += speed * angle.Sin();
 	}
 
+	void Thrust(const DVector3& vel)
+	{
+		Vel += vel;
+	}
+
 	void Vel3DFromAngle(DAngle angle, DAngle pitch, double speed)
 	{
 		double cospitch = pitch.Cos();
