@@ -373,8 +373,7 @@ static int P_CheckMissileRange (AActor *actor)
 	if (actor->MeleeState != nullptr && dist < actor->meleethreshold)
 		return false;	// From the Revenant: close enough for fist attack
 
-	if (actor->flags4 & MF4_MISSILEMORE) dist *= 0.5;
-	if (actor->flags4 & MF4_MISSILEEVENMORE) dist *= 0.125;
+	dist *= actor->missilechancemult;
 
 	int mmc = int(actor->MinMissileChance * G_SkillProperty(SKILLP_Aggressiveness));
 	return pr_checkmissilerange() >= min(int(dist), mmc);
@@ -465,6 +464,12 @@ static int P_IsUnderDamage(AActor* actor)
 				dir |= cl->getDirection();
 		}
 		// Q: consider crushing 3D floors too?
+		// [inkoalawetrust] Check for sectors that can harm the actor.
+		if (!(actor->flags9 & MF9_NOSECTORDAMAGE) && seclist->m_sector->damageamount > 0)
+		{
+			if (seclist->m_sector->MoreFlags & SECMF_HARMINAIR || actor->isAtZ(seclist->m_sector->LowestFloorAt(actor)) || actor->waterlevel)
+				return (actor->player || (actor->player == nullptr && seclist->m_sector->MoreFlags & SECMF_HURTMONSTERS))  ? -1 : 0;
+		}
 	}
 	return dir;
 }
@@ -1304,6 +1309,161 @@ int P_IsVisible(AActor *lookee, AActor *other, INTBOOL allaround, FLookExParams
 	return P_CheckSight(lookee, other, SF_SEEPASTSHOOTABLELINES);
 }
 
+bool isTargetablePlayer(AActor *actor, player_t *player, INTBOOL allaround, void* lookparams)
+{
+	FLookExParams* params = (FLookExParams*)lookparams;
+
+	if (!(player->mo->flags & MF_SHOOTABLE))
+		return false;			// not shootable (observer or dead)
+
+	if (actor->IsFriend(player->mo))
+		return false;			// same +MF_FRIENDLY, ignore
+
+	if (player->cheats & CF_NOTARGET)
+		return false;			// no target
+
+	if (player->health <= 0)
+		return false;			// dead
+
+	if (!P_IsVisible(actor, player->mo, allaround, params))
+		return false;			// out of sight
+
+	// [RC] Well, let's let special monsters with this flag active be able to see
+	// the player then, eh?
+	if (!(actor->flags6 & MF6_SEEINVISIBLE))
+	{
+		if ((player->mo->flags & MF_SHADOW && !(actor->Level->i_compatflags & COMPATF_INVISIBILITY)) ||
+			player->mo->flags3 & MF3_GHOST)
+		{
+			if (player->mo->Distance2D(actor) > 128 && player->mo->Vel.XY().LengthSquared() < 5 * 5)
+			{ // Player is sneaking - can't detect
+				return false;
+			}
+			if (pr_lookforplayers() < 225)
+			{ // Player isn't sneaking, but still didn't detect
+				return false;
+			}
+		}
+	}
+
+	return true;
+}
+
+bool ValidEnemyInBlock(AActor* lookee, AActor* other, void* lookparams)
+{
+	FLookExParams* params = (FLookExParams*)lookparams;
+
+	if (!(other->flags & MF_SHOOTABLE))
+		return false;			// not shootable (observer or dead)
+
+	if (other == lookee)
+		return false;			// is self
+
+	if (other->health <= 0)
+		return false;			// dead
+
+	if (other->flags2 & MF2_DORMANT)
+		return false;			// don't target dormant things
+
+	if (!(other->flags3 & MF3_ISMONSTER))
+		return false;			// don't target it if it isn't a monster (could be a barrel)
+
+	if (other->flags7 & MF7_NEVERTARGET)
+		return false;
+
+	bool keepChecking = false;
+	if (lookee->flags & MF_FRIENDLY)
+	{
+		if (other->flags & MF_FRIENDLY)
+		{
+			if (!lookee->IsFriend(other))
+			{
+				// This is somebody else's friend, so go after it
+				keepChecking = true;
+			}
+			else if (other->target != NULL && !(other->target->flags & MF_FRIENDLY))
+			{
+				other = other->target;
+				if (!(other->flags & MF_SHOOTABLE) ||
+					other->health <= 0 ||
+					(other->flags2 & MF2_DORMANT))
+				{
+					return false;
+				}
+			}
+		}
+		else
+		{
+			keepChecking = true;
+		}
+	}
+	else if (lookee->flags8 & MF8_SEEFRIENDLYMONSTERS && other->flags & MF_FRIENDLY)
+	{
+		keepChecking = true;
+	}
+
+	// [MBF] If the monster is already engaged in a one-on-one attack
+	// with a healthy friend, don't attack around 60% the time.
+
+	// [GrafZahl] This prevents friendlies from attacking all the same 
+	// target.
+
+	if (keepChecking)
+	{
+		AActor* targ = other->target;
+		if (targ && targ->target == other && pr_skiptarget() > 100 && lookee->IsFriend(targ) &&
+			targ->health * 2 >= targ->SpawnHealth())
+		{
+			return false;
+		}
+	}
+
+	// [KS] Hey, shouldn't there be a check for MF3_NOSIGHTCHECK here?
+
+	if (!keepChecking || !P_IsVisible(lookee, other, true, params))
+		return false;			// out of sight
+
+	return true;
+}
+
+//============================================================================
+//
+// LookForEnemiesEx
+//
+// [inkoalawetrust] Return a script array of all valid enemies of the caller
+// in range. For ZScript.
+//
+//============================================================================
+
+DEFINE_ACTION_FUNCTION(AActor, LookForEnemiesEx)
+{
+	PARAM_SELF_PROLOGUE(AActor);
+	PARAM_OUTPOINTER(targets,TArray<AActor*>);
+	PARAM_FLOAT(range);
+	PARAM_BOOL(noPlayers);
+	PARAM_BOOL(allaround);
+	PARAM_POINTER(params, FLookExParams);
+
+	if (targets == nullptr)
+		ThrowAbortException(X_WRITE_NIL,"No targets array passed");
+
+	if (range == -1)
+		range = self->friendlyseeblocks * FBlockmap::MAPBLOCKUNITS;
+
+	FPortalGroupArray check(FPortalGroupArray::PGA_Full3d);
+	FMultiBlockThingsIterator it(check, self, range, false);
+	FMultiBlockThingsIterator::CheckResult cres;
+
+	while (it.Next(&cres))
+	{
+		if (cres.thing->player == nullptr && ValidEnemyInBlock(cres.thing, self, params) ||
+			!noPlayers && cres.thing->player && isTargetablePlayer(self, cres.thing->player, allaround, params))
+			targets->Push(cres.thing);
+	}
+
+	ACTION_RETURN_INT(targets->Size());
+}
+
 //---------------------------------------------------------------------------
 //
 // FUNC P_LookForMonsters
@@ -1547,80 +1707,10 @@ AActor *LookForEnemiesInBlock (AActor *lookee, int index, void *extparam)
 	
 	for (block = lookee->Level->blockmap.blocklinks[index]; block != NULL; block = block->NextActor)
 	{
-		link = block->Me;
-
-        if (!(link->flags & MF_SHOOTABLE))
-			continue;			// not shootable (observer or dead)
-
-		if (link == lookee)
+		if (!ValidEnemyInBlock(lookee, block->Me, params))
 			continue;
 
-		if (link->health <= 0)
-			continue;			// dead
-
-		if (link->flags2 & MF2_DORMANT)
-			continue;			// don't target dormant things
-
-		if (!(link->flags3 & MF3_ISMONSTER))
-			continue;			// don't target it if it isn't a monster (could be a barrel)
-
-		if (link->flags7 & MF7_NEVERTARGET)
-			continue;
-
-		other = NULL;
-		if (lookee->flags & MF_FRIENDLY)
-		{
-			if (link->flags & MF_FRIENDLY)
-			{
-				if (!lookee->IsFriend(link))
-				{
-					// This is somebody else's friend, so go after it
-					other = link;
-				}
-				else if (link->target != NULL && !(link->target->flags & MF_FRIENDLY))
-				{
-					other = link->target;
-					if (!(other->flags & MF_SHOOTABLE) ||
-						other->health <= 0 ||
-						(other->flags2 & MF2_DORMANT))
-					{
-						other = NULL;;
-					}
-				}
-			}
-			else
-			{
-				other = link;
-			}
-		}
-		else if (lookee->flags8 & MF8_SEEFRIENDLYMONSTERS && link->flags & MF_FRIENDLY)
-		{
-			other = link;
-		}
-
-		// [MBF] If the monster is already engaged in a one-on-one attack
-		// with a healthy friend, don't attack around 60% the time.
-		
-		// [GrafZahl] This prevents friendlies from attacking all the same 
-		// target.
-		
-		if (other)
-		{
-			AActor *targ = other->target;
-			if (targ && targ->target == other && pr_skiptarget() > 100 && lookee->IsFriend (targ) &&
-				targ->health*2 >= targ->SpawnHealth())
-			{
-				continue;
-			}
-		}
-
-		// [KS] Hey, shouldn't there be a check for MF3_NOSIGHTCHECK here?
-
-		if (other == NULL || !P_IsVisible (lookee, other, true, params))
-			continue;			// out of sight
-
-
-		return other;
+		return block->Me;
 	}
 	return NULL;
 }
@@ -1820,46 +1910,8 @@ int P_LookForPlayers (AActor *actor, INTBOOL allaround, FLookExParams *params)
 
 		player = actor->Level->Players[pnum];
 
-		if (!(player->mo->flags & MF_SHOOTABLE))
-			continue;			// not shootable (observer or dead)
-
-		if (actor->IsFriend(player->mo))
-			continue;			// same +MF_FRIENDLY, ignore
-
-		if (player->cheats & CF_NOTARGET)
-			continue;			// no target
-
-		if (player->health <= 0)
-			continue;			// dead
-
-		if (!P_IsVisible (actor, player->mo, allaround, params))
-			continue;			// out of sight
-
-		// [SP] Deathmatch fixes - if we have MF_FRIENDLY we're definitely in deathmatch
-		// We're going to ignore our master, but go after his enemies.
-		if ( actor->flags & MF_FRIENDLY )
-		{
-			if ( actor->IsFriend(player->mo) )
-				continue;
-		}
-
-		// [RC] Well, let's let special monsters with this flag active be able to see
-		// the player then, eh?
-		if(!(actor->flags6 & MF6_SEEINVISIBLE)) 
-		{
-			if ((player->mo->flags & MF_SHADOW && !(actor->Level->i_compatflags & COMPATF_INVISIBILITY)) ||
-				player->mo->flags3 & MF3_GHOST)
-			{
-				if (player->mo->Distance2D (actor) > 128 && player->mo->Vel.XY().LengthSquared() < 5*5)
-				{ // Player is sneaking - can't detect
-					continue;
-				}
-				if (pr_lookforplayers() < 225)
-				{ // Player isn't sneaking, but still didn't detect
-					continue;
-				}
-			}
-		}
+		if (!isTargetablePlayer(actor, player, allaround, params))
+			continue;
 		
 		// [RH] Need to be sure the reactiontime is 0 if the monster is
 		//		leaving its goal to go after a player.
@@ -2139,7 +2191,7 @@ DEFINE_ACTION_FUNCTION(AActor, A_LookEx)
 
 	if (!(flags & LOF_NOSIGHTCHECK))
 	{
-		if (!P_LookForPlayers(self, true, &params))
+		if (!P_LookForPlayers(self, (self->flags4 & MF4_LOOKALLAROUND), &params)) // [RK] Account for the flag being set.
 			return 0;
 	}
 	else
@@ -2941,8 +2993,8 @@ void A_DoChaseNative(AActor * self, FState *melee, FState *missile, int flags)
 DEFINE_ACTION_FUNCTION_NATIVE(AActor, A_DoChase, A_DoChaseNative)
 {
 	PARAM_SELF_PROLOGUE(AActor);
-	PARAM_STATE(melee);
-	PARAM_STATE(missile);
+	PARAM_POINTER(melee, FState);
+	PARAM_POINTER(missile, FState);
 	PARAM_INT(flags);
 
 	A_DoChaseNative(self, melee, missile, flags);
