@@ -37,6 +37,7 @@
 #define FUDGEFACTOR		10
 
 static FRandom pr_teleport ("Teleport");
+static FRandom pr_playerteleport("PlayerTeleport");
 
 CVAR (Bool, telezoom, true, CVAR_ARCHIVE|CVAR_GLOBALCONFIG);
 
@@ -271,7 +272,7 @@ DEFINE_ACTION_FUNCTION(AActor, Teleport)
 //
 //-----------------------------------------------------------------------------
 
-AActor *FLevelLocals::SelectTeleDest (int tid, int tag, bool norandom)
+AActor *FLevelLocals::SelectTeleDest (int tid, int tag, bool norandom, bool isPlayer)
 {
 	AActor *searcher;
 
@@ -323,7 +324,8 @@ AActor *FLevelLocals::SelectTeleDest (int tid, int tag, bool norandom)
 		{
 			if (count != 1 && !norandom)
 			{
-				count = 1 + (pr_teleport() % count);
+				// Players get their own RNG seed to reduce likelihood of breaking prediction.
+				count = 1 + ((isPlayer ? pr_playerteleport() : pr_teleport()) % count);
 			}
 			searcher = NULL;
 			while (count > 0)
@@ -394,7 +396,7 @@ bool FLevelLocals::EV_Teleport (int tid, int tag, line_t *line, int side, AActor
 	{ // Don't teleport if hit back of line, so you can get out of teleporter.
 		return 0;
 	}
-	searcher = SelectTeleDest(tid, tag, predicting);
+	searcher = SelectTeleDest(tid, tag, false, thing->player != nullptr && thing->player->mo == thing);
 	if (searcher == NULL)
 	{
 		return false;
