@@ -60,11 +60,14 @@
 #include "hw_renderstate.h"
 #include "quaternion.h"
 
+#include "p_visualthinker.h"
+
 extern TArray<spritedef_t> sprites;
 extern TArray<spriteframe_t> SpriteFrames;
 extern uint32_t r_renderercaps;
 
 const float LARGE_VALUE = 1e19f;
+const float MY_SQRT2    = 1.41421356237309504880; // sqrt(2)
 
 EXTERN_CVAR(Bool, r_debug_disable_vis_filter)
 EXTERN_CVAR(Float, transsouls)
@@ -80,6 +83,8 @@ EXTERN_CVAR(Float, r_actorspriteshadowfadeheight)
 CVAR(Bool, gl_usecolorblending, true, CVAR_ARCHIVE | CVAR_GLOBALCONFIG)
 CVAR(Bool, gl_sprite_blend, false, CVAR_ARCHIVE | CVAR_GLOBALCONFIG);
 CVAR(Int, gl_spriteclip, 1, CVAR_ARCHIVE)
+CVAR(Bool, r_debug_nolimitanamorphoses, false, 0)
+CVAR(Float, r_spriteclipanamorphicminbias, 0.6, CVAR_ARCHIVE)
 CVAR(Float, gl_sclipthreshold, 10.0, CVAR_ARCHIVE)
 CVAR(Float, gl_sclipfactor, 1.8f, CVAR_ARCHIVE)
 CVAR(Int, gl_particles_style, 2, CVAR_ARCHIVE | CVAR_GLOBALCONFIG) // 0 = square, 1 = round, 2 = smooth
@@ -221,8 +226,15 @@ void HWSprite::DrawSprite(HWDrawInfo *di, FRenderState &state, bool translucent)
 		state.SetFog(0, 0);
 	}
 
+	int clampmode = CLAMP_XY;
+
+	if (texture && texture->isNoMipmap())
+	{
+		clampmode = CLAMP_XY_NOMIP;
+	}
+
 	uint32_t spritetype = actor? uint32_t(actor->renderflags & RF_SPRITETYPEMASK) : 0;
-	if (texture) state.SetMaterial(texture, UF_Sprite, (spritetype == RF_FACESPRITE) ? CTF_Expand : 0, CLAMP_XY, translation, OverrideShader);
+	if (texture) state.SetMaterial(texture, UF_Sprite, (spritetype == RF_FACESPRITE) ? CTF_Expand : 0, clampmode, translation, OverrideShader);
 	else if (!modelframe) state.EnableTexture(false);
 
 	//SetColor(lightlevel, rel, Colormap, trans);
@@ -355,11 +367,7 @@ void HandleSpriteOffsets(Matrix3x4 *mat, const FRotator *HW, FVector2 *offset, b
 
 bool HWSprite::CalculateVertices(HWDrawInfo* di, FVector3* v, DVector3* vp)
 {
-	float pixelstretch = 1.2;
-	if (actor && actor->Level)
-		pixelstretch = actor->Level->pixelstretch;
-	else if (particle && particle->subsector && particle->subsector->sector && particle->subsector->sector->Level)
-		pixelstretch = particle->subsector->sector->Level->pixelstretch;
+	float pixelstretch = di->Level->pixelstretch;
 
 	FVector3 center = FVector3((x1 + x2) * 0.5, (y1 + y2) * 0.5, (z1 + z2) * 0.5);
 	const auto& HWAngles = di->Viewpoint.HWAngles;
@@ -404,8 +412,9 @@ bool HWSprite::CalculateVertices(HWDrawInfo* di, FVector3* v, DVector3* vp)
 		&& (gl_billboard_mode == 1 || (actor && actor->renderflags & RF_FORCEXYBILLBOARD))));
 
 	const bool drawBillboardFacingCamera = hw_force_cambbpref ? gl_billboard_faces_camera :
-		(gl_billboard_faces_camera && (actor && !(actor->renderflags2 & RF2_BILLBOARDNOFACECAMERA)))
-		|| !!(actor && actor->renderflags2 & RF2_BILLBOARDFACECAMERA);
+		gl_billboard_faces_camera
+		|| ((actor && (!(actor->renderflags2 & RF2_BILLBOARDNOFACECAMERA) && (actor->renderflags2 & RF2_BILLBOARDFACECAMERA)))
+		|| (particle && particle->texture.isValid() && (!(particle->flags & SPF_NOFACECAMERA) && (particle->flags & SPF_FACECAMERA))));
 
 	// [Nash] has +ROLLSPRITE
 	const bool drawRollSpriteActor = (actor != nullptr && actor->renderflags & RF_ROLLSPRITE);
@@ -418,14 +427,14 @@ bool HWSprite::CalculateVertices(HWDrawInfo* di, FVector3* v, DVector3* vp)
 
 	// [Nash] is a flat sprite
 	const bool isWallSprite = (actor != nullptr) && (spritetype == RF_WALLSPRITE);
-	const bool useOffsets = (actor != nullptr) && !(actor->renderflags & RF_ROLLCENTER);
+	const bool useOffsets = ((actor != nullptr) && !(actor->renderflags & RF_ROLLCENTER)) || (particle && !(particle->flags & SPF_ROLLCENTER));
 
 	FVector2 offset = FVector2( offx, offy );
 	float xx = -center.X + x;
 	float yy = -center.Y + y;
 	float zz = -center.Z + z;
 	// [Nash] check for special sprite drawing modes
-	if (drawWithXYBillboard || isWallSprite)
+	if (drawWithXYBillboard || drawBillboardFacingCamera || isWallSprite)
 	{
 		mat.MakeIdentity();
 		mat.Translate(center.X, center.Z, center.Y); // move to sprite center
@@ -495,7 +504,7 @@ bool HWSprite::CalculateVertices(HWDrawInfo* di, FVector3* v, DVector3* vp)
 	}
 	else // traditional "Y" billboard mode
 	{
-		if (doRoll || !offset.isZero())
+		if (doRoll || !offset.isZero() || (actor && (actor->renderflags2 & RF2_ISOMETRICSPRITES)))
 		{
 			mat.MakeIdentity();
 
@@ -517,6 +526,16 @@ bool HWSprite::CalculateVertices(HWDrawInfo* di, FVector3* v, DVector3* vp)
 				mat.Translate(-center.X, -center.Z, -center.Y);
 			}
 
+			if (actor && (actor->renderflags2 & RF2_ISOMETRICSPRITES) && di->Viewpoint.IsOrtho())
+			{
+				float angleRad = (FAngle::fromDeg(270.) - HWAngles.Yaw).Radians();
+				mat.Translate(center.X, center.Z, center.Y);
+				mat.Translate(0.0, z2 - center.Z, 0.0);
+				mat.Rotate(-sin(angleRad), 0, cos(angleRad), -actor->isotheta);
+				mat.Translate(0.0, center.Z - z2, 0.0);
+				mat.Translate(-center.X, -center.Z, -center.Y);
+			}
+
 			v[0] = mat * FVector3(x1, z1, y1);
 			v[1] = mat * FVector3(x2, z1, y2);
 			v[2] = mat * FVector3(x1, z2, y1);
@@ -910,6 +929,7 @@ void HWSprite::Process(HWDrawInfo *di, AActor* thing, sector_t * sector, area_t
 	{
 		bool mirror = false;
 		DAngle ang = (thingpos - vp.Pos).Angle();
+		if (di->Viewpoint.IsOrtho()) ang = vp.Angles.Yaw;
 		FTextureID patch;
 		// [ZZ] add direct picnum override
 		if (isPicnumOverride)
@@ -1003,14 +1023,39 @@ void HWSprite::Process(HWDrawInfo *di, AActor* thing, sector_t * sector, area_t
 		if (thing->renderflags & RF_SPRITEFLIP) // [SP] Flip back
 			thing->renderflags ^= RF_XFLIP;
 
-		r.Scale(sprscale.X, isSpriteShadow ? sprscale.Y * 0.15 : sprscale.Y);
+		// If sprite is isometric, do both vertical scaling and partial rotation to face the camera to compensate for Y-billboarding.
+		// Using just rotation (about z=0) might cause tall+slender (high aspect ratio) sprites to clip out of collision box
+		// at the top and clip into whatever is behind them from the viewpoint's perspective. - [DVR]
+		thing->isoscaleY = 1.0;
+		thing->isotheta = vp.HWAngles.Pitch.Degrees();
+		if (thing->renderflags2 & RF2_ISOMETRICSPRITES)
+		{
+			float floordist = thing->radius * vp.floordistfact;
+			floordist -= 0.5 * r.width * vp.cotfloor;
+			float sineisotheta = floordist / r.height;
+			double scl = g_sqrt( 1.0 + sineisotheta * sineisotheta - 2.0 * vp.PitchSin * sineisotheta );
+			if ((thing->radius > 0.0) && (scl > fabs(vp.PitchCos)))
+			{
+				thing->isoscaleY = scl / ( fabs(vp.PitchCos) > 0.01 ? fabs(vp.PitchCos) : 0.01 );
+				thing->isotheta = 180.0 * asin( sineisotheta / thing->isoscaleY ) / M_PI;
+			}
+		}
+
+		r.Scale(sprscale.X, isSpriteShadow ? sprscale.Y * 0.15 * thing->isoscaleY : sprscale.Y * thing->isoscaleY);
+
+		if (((thing->renderflags & RF_ROLLSPRITE) || (thing->renderflags2 & RF2_SQUAREPIXELS)) && !(thing->renderflags2 & RF2_STRETCHPIXELS))
+		{
+			double ps = di->Level->pixelstretch;
+			double mult = 1.0 / sqrt(ps); // shrink slightly
+			r.Scale(mult * ps, mult);
+		}
 
 		float rightfac = -r.left;
 		float leftfac = rightfac - r.width;
 		z1 = z - r.top;
 		z2 = z1 - r.height;
 
-		float spriteheight = sprscale.Y * r.height;
+		float spriteheight = sprscale.Y * r.height * thing->isoscaleY;
 
 		// Tests show that this doesn't look good for many decorations and corpses
 		if (spriteheight > 0 && gl_spriteclip > 0 && (thing->renderflags & RF_SPRITETYPEMASK) == RF_FACESPRITE)
@@ -1029,6 +1074,13 @@ void HWSprite::Process(HWDrawInfo *di, AActor* thing, sector_t * sector, area_t
 			x2 = x - viewvecY*rightfac;
 			y1 = y + viewvecX*leftfac;
 			y2 = y + viewvecX*rightfac;
+			if (thing->renderflags2 & RF2_ISOMETRICSPRITES) // If sprites are drawn from an isometric perspective
+			{
+				x1 -= viewvecX * thing->radius * MY_SQRT2;
+				x2 -= viewvecX * thing->radius * MY_SQRT2;
+				y1 -= viewvecY * thing->radius * MY_SQRT2;
+				y2 -= viewvecY * thing->radius * MY_SQRT2;
+			}
 			break;
 		}
 		case RF_FLATSPRITE:
@@ -1069,8 +1121,59 @@ void HWSprite::Process(HWDrawInfo *di, AActor* thing, sector_t * sector, area_t
 	}
 
 	depth = (float)((x - vp.Pos.X) * vp.TanCos + (y - vp.Pos.Y) * vp.TanSin);
+	if(thing->renderflags2 & RF2_ISOMETRICSPRITES) depth = depth * vp.PitchCos - vp.PitchSin * z2; // Helps with stacking actors with small xy offsets
 	if (isSpriteShadow) depth += 1.f/65536.f; // always sort shadows behind the sprite.
 
+	if (gl_spriteclip == -1 && (thing->renderflags & RF_SPRITETYPEMASK) == RF_FACESPRITE) // perform anamorphosis
+	{
+		float minbias = r_spriteclipanamorphicminbias;
+		minbias = clamp(minbias, 0.3f, 1.0f);
+
+		float btm = thing->Sector->floorplane.ZatPoint(thing) - thing->Floorclip;
+		float top = thing->Sector->ceilingplane.ZatPoint(thingpos);
+
+		float vbtm = thing->Sector->floorplane.ZatPoint(vp.Pos);
+		float vtop = thing->Sector->ceilingplane.ZatPoint(vp.Pos);
+
+		float vpx = vp.Pos.X;
+		float vpy = vp.Pos.Y;
+		float vpz = vp.Pos.Z;
+
+		float tpx = thingpos.X;
+		float tpy = thingpos.Y;
+		float tpz = thingpos.Z;
+
+		if (!(r_debug_nolimitanamorphoses))
+		{
+			// this should help prevent clipping through walls ...
+			float objradiusbias = 1.f - thing->radius / sqrt((vpx - tpx) * (vpx - tpx) + (vpy - tpy) * (vpy - tpy));
+			minbias = max(minbias, objradiusbias);
+		}
+
+		float bintersect, tintersect;
+		if (z2 < vpz && vbtm < vpz)
+			bintersect = min((btm - vpz) / (z2 - vpz), (vbtm - vpz) / (z2 - vpz));
+		else
+			bintersect = 1.0;
+
+		if (z1 > vpz && vtop > vpz)
+			tintersect = min((top - vpz) / (z1 - vpz), (vtop - vpz) / (z1 - vpz));
+		else
+			tintersect = 1.0;
+
+		if (thing->waterlevel >= 1 && thing->waterlevel <= 2)
+			bintersect = tintersect = 1.0f;
+
+		float spbias = clamp(min(bintersect, tintersect), minbias, 1.0f);
+		float vpbias = 1.0 - spbias;
+		x1 = x1 * spbias + vpx * vpbias;
+		y1 = y1 * spbias + vpy * vpbias;
+		z1 = z1 * spbias + vpz * vpbias;
+		x2 = x2 * spbias + vpx * vpbias;
+		y2 = y2 * spbias + vpy * vpbias;
+		z2 = z2 * spbias + vpz * vpbias;		
+	}
+
 	// light calculation
 
 	bool enhancedvision = false;
@@ -1289,6 +1392,7 @@ void HWSprite::Process(HWDrawInfo *di, AActor* thing, sector_t * sector, area_t
 	{
 		lightlist = nullptr;
 	}
+
 	PutSprite(di, hw_styleflags != STYLEHW_Solid);
 	rendered_sprites++;
 }
@@ -1312,7 +1416,7 @@ void HWSprite::ProcessParticle(HWDrawInfo *di, particle_t *particle, sector_t *s
 	foglevel = (uint8_t)clamp<short>(sector->lightlevel, 0, 255);
 
 	trans = particle->alpha;
-	OverrideShader = 0;
+	OverrideShader = (particle->flags & SPF_ALLOWSHADERS) ? -1 : 0;
 	modelframe = nullptr;
 	texture = nullptr;
 	topclip = LARGE_VALUE;
@@ -1447,7 +1551,11 @@ void HWSprite::ProcessParticle(HWDrawInfo *di, particle_t *particle, sector_t *s
 		else factor = 1 / 7.f;
 		float scalefac=particle->size * factor;
 
-		float viewvecX = vp.ViewVector.X * scalefac;
+		float ps = di->Level->pixelstretch;
+
+		scalefac /= sqrt(ps); // shrink it slightly to account for the stretch
+
+		float viewvecX = vp.ViewVector.X * scalefac * ps;
 		float viewvecY = vp.ViewVector.Y * scalefac;
 
 		x1=x+viewvecY;
@@ -1493,7 +1601,7 @@ void HWSprite::AdjustVisualThinker(HWDrawInfo* di, DVisualThinker* spr, sector_t
 			? TexAnim.UpdateStandaloneAnimation(spr->PT.animData, di->Level->maptime + timefrac)
 			: spr->PT.texture, !custom_anim);
 
-	if (spr->bDontInterpolate)
+	if (spr->flags & VTF_DontInterpolate)
 		timefrac = 0.;
 
 	FVector3 interp = spr->InterpolatedPosition(timefrac);
@@ -1517,12 +1625,18 @@ void HWSprite::AdjustVisualThinker(HWDrawInfo* di, DVisualThinker* spr, sector_t
 	auto r = spi.GetSpriteRect();
 	r.Scale(spr->Scale.X, spr->Scale.Y);
 
-	if (spr->bXFlip)	
+	if ((spr->PT.flags & SPF_ROLL) && !(spr->PT.flags & SPF_STRETCHPIXELS))
+	{
+		double ps = di->Level->pixelstretch;
+		double mult = 1.0 / sqrt(ps); // shrink slightly
+		r.Scale(mult * ps, mult);
+	}
+	if (spr->flags & VTF_FlipX)
 	{
 		std::swap(ul,ur);
 		r.left = -r.width - r.left;	// mirror the sprite's x-offset
 	}
-	if (spr->bYFlip)	std::swap(vt,vb);
+	if (spr->flags & VTF_FlipY)	std::swap(vt,vb);
 
 	float viewvecX = vp.ViewVector.X;
 	float viewvecY = vp.ViewVector.Y;
