@@ -869,6 +869,34 @@ DEFINE_ACTION_FUNCTION_NATIVE(_Sector, SetXOffset, SetXOffset)
 	 ACTION_RETURN_INT(self->GetLightLevel());
  }
 
+ static void SetPlaneReflectivity(sector_t* self, int pos, double val)
+ {
+	 if (pos < 0 || pos > 1) ThrowAbortException(X_ARRAY_OUT_OF_BOUNDS, "pos must be either 0 or 1");
+	 self->SetPlaneReflectivity(pos, val);
+ }
+
+ DEFINE_ACTION_FUNCTION_NATIVE(_Sector, SetPlaneReflectivity, SetPlaneReflectivity)
+ {
+	 PARAM_SELF_STRUCT_PROLOGUE(sector_t);
+	 PARAM_INT(pos);
+	 PARAM_FLOAT(val)
+	 SetPlaneReflectivity(self, pos, val);
+	 return 0;
+ }
+
+ static double GetPlaneReflectivity(sector_t* self, int pos)
+ {
+	 if (pos < 0 || pos > 1) ThrowAbortException(X_ARRAY_OUT_OF_BOUNDS, "pos must be either 0 or 1");
+	 return self->GetPlaneReflectivity(pos);
+ }
+
+ DEFINE_ACTION_FUNCTION_NATIVE(_Sector, GetPlaneReflectivity, GetPlaneReflectivity)
+ {
+	 PARAM_SELF_STRUCT_PROLOGUE(sector_t);
+	 PARAM_INT(pos);
+	 ACTION_RETURN_FLOAT(GetPlaneReflectivity(self, pos));
+ }
+
  static int PortalBlocksView(sector_t *self, int pos)
  {
 	 return self->PortalBlocksView(pos);
@@ -1715,7 +1743,7 @@ DEFINE_ACTION_FUNCTION_NATIVE(_Sector, SetXOffset, SetXOffset)
 	 PARAM_SELF_STRUCT_PROLOGUE(FLevelLocals);
 	 PARAM_NAME(seq);
 	 PARAM_INT(state);
-	 G_StartSlideshow(self, seq);
+	 G_StartSlideshow(self, seq, state);
 	 return 0;
  }
 
@@ -2263,7 +2291,13 @@ void FormatMapName(FLevelLocals *self, int cr, FString *result)
 	bool ishub = (cluster != nullptr && (cluster->flags & CLUSTER_HUB));
 
 	*result = "";
-	if (am_showmaplabel == 1 || (am_showmaplabel == 2 && !ishub))
+	// If a label is specified, use it uncontitionally here.
+	if (self->info->MapLabel.IsNotEmpty())
+	{
+		if (self->info->MapLabel.Compare("*"))
+			*result << self->info->MapLabel << ": ";
+	}
+	else if (am_showmaplabel == 1 || (am_showmaplabel == 2 && !ishub))
 	{
 		*result << self->MapName << ": ";
 	}
@@ -2635,7 +2669,7 @@ DEFINE_ACTION_FUNCTION(_Console, MidPrint)
 	PARAM_STRING(text);
 	PARAM_BOOL(bold);
 
-	const char* txt = text[0] == '$' ? GStrings(&text[1]) : text.GetChars();
+	const char* txt = text[0] == '$' ? GStrings.GetString(&text[1]) : text.GetChars();
 	C_MidPrint(fnt, txt, bold);
 	return 0;
 }
@@ -2767,6 +2801,7 @@ DEFINE_FIELD_X(LevelInfo, level_info_t, Music)
 DEFINE_FIELD_X(LevelInfo, level_info_t, LightningSound)
 DEFINE_FIELD_X(LevelInfo, level_info_t, LevelName)
 DEFINE_FIELD_X(LevelInfo, level_info_t, AuthorName)
+DEFINE_FIELD_X(LevelInfo, level_info_t, MapLabel)
 DEFINE_FIELD_X(LevelInfo, level_info_t, musicorder)
 DEFINE_FIELD_X(LevelInfo, level_info_t, skyspeed1)
 DEFINE_FIELD_X(LevelInfo, level_info_t, skyspeed2)
