@@ -1851,16 +1851,18 @@ DEFINE_ACTION_FUNCTION(AActor, BouncePlane)
 {
 	PARAM_SELF_PROLOGUE(AActor);
 	PARAM_POINTER(plane, secplane_t);
+	PARAM_BOOL(is3DFloor);
 
-	ACTION_RETURN_BOOL(self->FloorBounceMissile(*plane));
+	ACTION_RETURN_BOOL(self->FloorBounceMissile(*plane, is3DFloor));
 }
 
 DEFINE_ACTION_FUNCTION(AActor, PlayBounceSound)
 {
 	PARAM_SELF_PROLOGUE(AActor);
 	PARAM_BOOL(onFloor);
+	PARAM_FLOAT(volume);
 
-	self->PlayBounceSound(onFloor);
+	self->PlayBounceSound(onFloor, volume);
 	return 0;
 }
 
@@ -2005,6 +2007,7 @@ DEFINE_FIELD(AActor, strafecount)
 DEFINE_FIELD(AActor, target)
 DEFINE_FIELD(AActor, master)
 DEFINE_FIELD(AActor, tracer)
+DEFINE_FIELD(AActor, damagesource)
 DEFINE_FIELD(AActor, LastHeard)
 DEFINE_FIELD(AActor, lastenemy)
 DEFINE_FIELD(AActor, LastLookActor)
@@ -2035,6 +2038,7 @@ DEFINE_FIELD(AActor, DamageType)
 DEFINE_FIELD(AActor, DamageTypeReceived)
 DEFINE_FIELD(AActor, FloatBobPhase)
 DEFINE_FIELD(AActor, FloatBobStrength)
+DEFINE_FIELD(AActor, FloatBobFactor)
 DEFINE_FIELD(AActor, RipperLevel)
 DEFINE_FIELD(AActor, RipLevelMin)
 DEFINE_FIELD(AActor, RipLevelMax)
@@ -2042,6 +2046,7 @@ DEFINE_FIELD(AActor, Species)
 DEFINE_FIELD(AActor, alternative)
 DEFINE_FIELD(AActor, goal)
 DEFINE_FIELD(AActor, MinMissileChance)
+DEFINE_FIELD(AActor, missilechancemult)
 DEFINE_FIELD(AActor, LastLookPlayerNumber)
 DEFINE_FIELD(AActor, SpawnFlags)
 DEFINE_FIELD(AActor, meleethreshold)
