@@ -154,7 +154,7 @@ static FString LookupMusic(const char* musicname, int& order)
 		// handle dehacked replacement.
 		// Any music name defined this way needs to be prefixed with 'D_' because
 		// Doom.exe does not contain the prefix so these strings don't either.
-		const char* mus_string = GStrings[musicname + 1];
+		const char* mus_string = GStrings.CheckString(musicname + 1);
 		if (mus_string != nullptr)
 		{
 			DEH_Music << "D_" << mus_string;
@@ -379,7 +379,7 @@ void S_InitData()
 
 void S_SoundPitch(int channel, EChanFlags flags, FSoundID sound_id, float volume, float attenuation, float pitch, float startTime)
 {
-	soundEngine->StartSound(SOURCE_None, nullptr, nullptr, channel, flags, sound_id, volume, attenuation, 0, pitch);
+	soundEngine->StartSound(SOURCE_None, nullptr, nullptr, channel, flags, sound_id, volume, attenuation, nullptr, pitch, startTime);
 }
 
 void S_Sound(int channel, EChanFlags flags, FSoundID sound_id, float volume, float attenuation)
@@ -414,6 +414,29 @@ DEFINE_ACTION_FUNCTION(DObject, S_StartSound)
 	return 0;
 }
 
+static void S_StartSoundAt(double x, double y, double z, int sound_id, int channel, int flags, double volume, double attenuation, double pitch, double startTime)
+{
+	FVector3 pos = { (float)x, (float)z, (float)y };
+	soundEngine->StartSound(SOURCE_Unattached, nullptr, &pos, channel, EChanFlags::FromInt(flags), FSoundID::fromInt(sound_id), (float)volume, (float)attenuation, nullptr, (float)pitch, (float)startTime);
+}
+
+DEFINE_ACTION_FUNCTION_NATIVE(DObject, S_StartSoundAt, S_StartSoundAt)
+{
+	PARAM_PROLOGUE;
+	PARAM_FLOAT(x);
+	PARAM_FLOAT(y);
+	PARAM_FLOAT(z);
+	PARAM_INT(id);
+	PARAM_INT(channel);
+	PARAM_INT(flags);
+	PARAM_FLOAT(volume);
+	PARAM_FLOAT(attn);
+	PARAM_FLOAT(pitch);
+	PARAM_FLOAT(startTime);
+	S_StartSoundAt(x, y, z, id, channel, flags, volume, attn, pitch, startTime);
+	return 0;
+}
+
 
 //==========================================================================
 //
@@ -1163,7 +1186,7 @@ TArray<uint8_t> DoomSoundEngine::ReadSound(int lumpnum)
 // This is overridden to use a synchronized RNG.
 // 
 //==========================================================================
-static FRandom pr_randsound("RandSound");
+static FCRandom pr_randsound("RandSound");
 
 FSoundID DoomSoundEngine::PickReplacement(FSoundID refid)
 {
